package Contact

import (
	"context"

	"wechat-api/ent/contact"
	"wechat-api/ent/predicate"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

    "github.com/suyuan32/simple-admin-common/msg/errormsg"

	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"github.com/zeromicro/go-zero/core/logx"
)

type GetContactListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetContactListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetContactListLogic {
	return &GetContactListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetContactListLogic) GetContactList(req *types.ContactListReq) (*types.ContactListResp, error) {
	var predicates []predicate.Contact
	if req.WxWxid != nil {
		predicates = append(predicates, contact.WxWxidContains(*req.WxWxid))
	}
	if req.Wxid != nil {
		predicates = append(predicates, contact.WxidContains(*req.Wxid))
	}
	if req.Account != nil {
		predicates = append(predicates, contact.AccountContains(*req.Account))
	}
	data, err := l.svcCtx.DB.Contact.Query().Where(predicates...).Page(l.ctx, req.Page, req.PageSize)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	resp := &types.ContactListResp{}
	resp.Msg = errormsg.Success
	resp.Data.Total = data.PageDetails.Total

	for _, v := range data.List {
		resp.Data.Data = append(resp.Data.Data,
		types.ContactInfo{
			BaseIDInfo:    types.BaseIDInfo{
				Id:          &v.ID,
				CreatedAt:    pointy.GetPointer(v.CreatedAt.UnixMilli()),
				UpdatedAt:    pointy.GetPointer(v.UpdatedAt.UnixMilli()),
            },
			Status:	&v.Status,
			WxWxid:	&v.WxWxid,
			Type:	&v.Type,
			Wxid:	&v.Wxid,
			Account:	&v.Account,
			Nickname:	&v.Nickname,
			Markname:	&v.Markname,
			Headimg:	&v.Headimg,
			Sex:	&v.Sex,
			Starrole:	&v.Starrole,
			Dontseeit:	&v.Dontseeit,
			Dontseeme:	&v.Dontseeme,
			Lag:	&v.Lag,
			Gid:	&v.Gid,
			Gname:	&v.Gname,
			V3:	&v.V3,
		})
	}

	return resp, nil
}
